package com.qmth.paper.library.business.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.paper.library.business.bean.result.SelectResult;
import com.qmth.paper.library.business.entity.ExamCourse;
import com.qmth.paper.library.business.mapper.ExamCourseMapper;
import com.qmth.paper.library.business.service.ExamCourseService;
import com.qmth.paper.library.common.entity.ExamStudent;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * <p>
 * 考生课程表 服务实现类
 * </p>
 *
 * @author xf
 * @since 2024-03-11
 */
@Service
public class ExamCourseServiceImpl extends ServiceImpl<ExamCourseMapper, ExamCourse> implements ExamCourseService {

    @Transactional
    @Override
    public void saveStudentCourse(Long examId, List<ExamStudent> basicStudentList) {
        this.removeByExamId(examId);
        Map<String, String> stringMap = basicStudentList.stream().collect(Collectors.toMap(ExamStudent::getCourseCode, ExamStudent::getCourseName, (oldValue, newValue) -> oldValue));
        List<ExamCourse> examCourses = new ArrayList<>();
        for (Map.Entry<String, String> entry : stringMap.entrySet()) {
            examCourses.add(new ExamCourse(examId, entry.getKey(), entry.getValue()));
        }
        this.saveBatch(examCourses);
    }

    @Override
    public void removeByExamId(Long examId) {
        QueryWrapper<ExamCourse> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(ExamCourse::getExamId, examId);
        this.remove(queryWrapper);
    }

    @Override
    public List<SelectResult> listConditionByExamId(Long examId) {

        List<SelectResult> selectResults = new ArrayList<>();
        for (ExamCourse examCourse : this.listByExamId(examId)) {
            SelectResult selectResult = new SelectResult();
            selectResult.setCode(examCourse.getCourseCode());
            selectResult.setName(examCourse.getCourseName());
            selectResults.add(selectResult);
        }
        return selectResults;
    }

    @Override
    public List<ExamCourse> listByExamId(Long examId) {
        QueryWrapper<ExamCourse> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(ExamCourse::getExamId, examId)
                .orderByDesc(ExamCourse::getId);
        return this.list(queryWrapper);
    }
}
