package com.qmth.paper.library.business.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.boot.tools.excel.ExcelReader;
import com.qmth.boot.tools.excel.enums.ExcelType;
import com.qmth.boot.tools.excel.model.DataMap;
import com.qmth.paper.library.business.mapper.ExamStudentMapper;
import com.qmth.paper.library.business.service.ExamCourseService;
import com.qmth.paper.library.business.service.ExamStudentService;
import com.qmth.paper.library.business.service.PaperLibraryService;
import com.qmth.paper.library.business.service.PaperScanTaskService;
import com.qmth.paper.library.common.bean.dto.syssetting.SimpleObject;
import com.qmth.paper.library.common.contant.SysSettingConstant;
import com.qmth.paper.library.common.contant.SystemConstant;
import com.qmth.paper.library.common.entity.ExamStudent;
import com.qmth.paper.library.common.entity.SysUser;
import com.qmth.paper.library.common.enums.ExceptionResultEnum;
import com.qmth.paper.library.common.enums.RecognitionTypeEnum;
import com.qmth.paper.library.common.enums.StoreTypeEnum;
import com.qmth.paper.library.common.service.CommonCacheService;
import com.qmth.paper.library.common.util.ServletUtil;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.io.IOException;
import java.util.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 学校id 服务实现类
 * </p>
 *
 * @author wangliang
 * @since 2021-08-02
 */
@Service
public class ExamStudentServiceImpl extends ServiceImpl<ExamStudentMapper, ExamStudent> implements ExamStudentService {
    @Resource
    private ExamCourseService examCourseService;
    @Resource
    private PaperLibraryService paperLibraryService;
    @Resource
    private PaperScanTaskService paperScanTaskService;
    @Resource
    private CommonCacheService commonCacheService;

    @Override
    public IPage<ExamStudent> findStudentPage(Long examId, String courseCode, String collegeName, String majorName, String clazzName, String queryParams, int pageNumber, int pageSize) {
        queryParams = SystemConstant.translateSpecificSign(queryParams);
        return this.baseMapper.findStudentPage(new Page<>(pageNumber, pageSize), examId, courseCode, collegeName, majorName, clazzName, queryParams);
    }

    @Override
    public void saveBasicStudent(ExamStudent basicStudent) {
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        // 验证
        ExamStudent byStudentCode = this.getByExamIdAndCourseCodeAndStudentCode(basicStudent.getExamId(), basicStudent.getCourseCode(), basicStudent.getStudentCode());
        if (basicStudent.getId() == null) {
            if (byStudentCode != null) {
                throw ExceptionResultEnum.ERROR.exception(String.format("学号[%s]在课程%s（%s）下已存在", basicStudent.getStudentCode(), basicStudent.getCourseName(), basicStudent.getCourseCode()));
            }
            byStudentCode.setId(SystemConstant.getDbUuid());
            byStudentCode.setSchoolId(sysUser.getSchoolId());
            byStudentCode.setCreateId(sysUser.getId());
            byStudentCode.setCreateTime(System.currentTimeMillis());
        } else {
            if (!basicStudent.getId().equals(byStudentCode.getId())) {
                throw ExceptionResultEnum.ERROR.exception(String.format("学号[%s]在课程%s（%s）下已存在", basicStudent.getStudentCode(), basicStudent.getCourseName(), basicStudent.getCourseCode()));
            }
            byStudentCode.setId(basicStudent.getId());
            byStudentCode.setEnable(true);
            byStudentCode.setUpdateId(sysUser.getId());
            byStudentCode.setUpdateTime(System.currentTimeMillis());
        }
        this.saveOrUpdate(byStudentCode);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public Boolean removeBasicStudentBatch(List<Long> ids) {
        if (CollectionUtils.isEmpty(ids)) {
            throw ExceptionResultEnum.ERROR.exception("请选择删除的数据");
        }
        List<ExamStudent> examStudentList = this.listByIds(ids);
        if (CollectionUtils.isNotEmpty(examStudentList)) {
            List<String> studentCodes = examStudentList.stream().filter(m -> m.getBindCount() > 0).map(ExamStudent::getStudentCode).collect(Collectors.toList());
            if (CollectionUtils.isNotEmpty(studentCodes)) {
                throw ExceptionResultEnum.ERROR.exception("有考生已绑定图片，学号：" + String.join(",", studentCodes));
            }
        }
        // 删除基础学生
        return this.removeByIds(ids);
    }

    @Override
    public ExamStudent getByExamIdAndCourseCodeAndStudentCode(Long examId, String courseCode, String studentCode) {
        QueryWrapper<ExamStudent> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(ExamStudent::getExamId, examId)
                .eq(ExamStudent::getCourseCode, courseCode)
                .eq(ExamStudent::getStudentCode, studentCode);
        return this.getOne(queryWrapper);
    }

    @Transactional
    @Override
    public void dataImport(Long semesterId, Long examId, MultipartFile file) {
        SysUser sysUser = (SysUser) ServletUtil.getRequestUser();
        // 校验是否设置了存储方式
        SimpleObject recognitionTypeSimpleObject = commonCacheService.getSysSetting(sysUser.getSchoolId()).get(SysSettingConstant.RECOGNITION_TYPE);
        if (StringUtils.isBlank(recognitionTypeSimpleObject.getValue())) {
            throw ExceptionResultEnum.ERROR.exception("识别方式未设置");
        }
        SimpleObject storeTypeSimpleObject = commonCacheService.getSysSetting(sysUser.getSchoolId()).get(SysSettingConstant.STORE_TYPE);
        if (StringUtils.isBlank(storeTypeSimpleObject.getValue())) {
            throw ExceptionResultEnum.ERROR.exception("文件存储方式未设置");
        }
        RecognitionTypeEnum recognitionType = RecognitionTypeEnum.valueOf(recognitionTypeSimpleObject.getValue());
        StoreTypeEnum storeType = StoreTypeEnum.valueOf(storeTypeSimpleObject.getValue());

        // 校验是否已扫描
        if (paperLibraryService.countByExamId(examId) > 0) {
            throw ExceptionResultEnum.ERROR.exception("已开始扫描，导入失败。请清除扫描数据后重试");
        }
        try {
            System.out.println("大小：" + file.getSize());
            ExcelReader excelReader = ExcelReader.create(ExcelType.XLSX, file.getInputStream(), 0);
//            List<String[]> dataMapList = excelReader.getDataArrayList();
            List<ExamStudent> basicStudentList = excelReader.getObjectList(ExamStudent.class);
            // 校验excel内容
            validStudentData(semesterId, examId, basicStudentList, storeType, sysUser);
            // 删除
            this.removeByExamId(examId);
            // 保存学生
            this.saveBatch(basicStudentList);
            // 保存课程
            examCourseService.saveStudentCourse(examId, basicStudentList);
            // 新建任务
            paperScanTaskService.addScanTask(examId, recognitionType, storeType, basicStudentList);
        } catch (IOException e) {
            throw new RuntimeException(e);
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

    @Override
    public void updateBindCount(Long studentId) {
        UpdateWrapper<ExamStudent> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().set(ExamStudent::getBindCount, paperLibraryService.countByStudentId(studentId))
                .eq(ExamStudent::getId, studentId);
        this.update(updateWrapper);
    }

    @Override
    public void removeByExamId(Long examId) {
        QueryWrapper<ExamStudent> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(ExamStudent::getExamId, examId);
        this.remove(queryWrapper);
    }

    /**
     * 学生保存公共方法
     */
    private void validStudentData(Long semesterId, Long examId, List<ExamStudent> basicStudentList, StoreTypeEnum storeType, SysUser sysUser) {
        Map<String, String> courseMap = new HashMap<>();
        Map<String, List<String>> studentCodeMap = new HashMap<>();
        for (ExamStudent basicStudent : basicStudentList) {
            StringJoiner errorMsg = new StringJoiner("；");
            if (StringUtils.isBlank(basicStudent.getStudentCode())) {
                errorMsg.add("学号必填");
            }
            if (StringUtils.isBlank(basicStudent.getStudentName())) {
                errorMsg.add("姓名必填");
            }
            if (StringUtils.isBlank(basicStudent.getCourseCode())) {
                errorMsg.add("课程代码必填");
            }
            if (StringUtils.isBlank(basicStudent.getCourseName())) {
                errorMsg.add("课程名称必填");
            }
            if (StoreTypeEnum.ROOM.equals(storeType) && StringUtils.isBlank(basicStudent.getExamRoom())) {
                errorMsg.add("考场必填");
            }
            if (StoreTypeEnum.CLASS.equals(storeType) && StringUtils.isBlank(basicStudent.getClassName())) {
                errorMsg.add("班级必填");
            }
            if (courseMap.containsKey(basicStudent.getCourseCode())) {
                if (!basicStudent.getCourseName().equals(courseMap.get(basicStudent.getCourseCode()))) {
                    errorMsg.add("课程代码已存在课程名称[" + courseMap.get(basicStudent.getCourseCode()) + "]");
                }
            }
            // 考生在同课程下重复
            if (studentCodeMap.containsKey(basicStudent.getCourseCode())) {
                List<String> studentCodeList = studentCodeMap.get(basicStudent.getCourseCode());
                if (CollectionUtils.isEmpty(studentCodeList)) {
                    studentCodeMap.put(basicStudent.getCourseCode(), new ArrayList<>());
                } else {
                    if (studentCodeList.contains(basicStudent.getStudentCode())) {
                        errorMsg.add("学号在同课程下已存在");
                    } else {
                        studentCodeList.add(basicStudent.getStudentCode());
                        studentCodeMap.put(basicStudent.getCourseCode(), studentCodeList);
                    }
                }
            }

            basicStudent.setId(SystemConstant.getDbUuid());
            basicStudent.setSchoolId(sysUser.getSchoolId());
            basicStudent.setSemesterId(semesterId);
            basicStudent.setExamId(examId);
            basicStudent.setCreateId(sysUser.getId());
            basicStudent.setCreateTime(System.currentTimeMillis());
        }
    }
}
